{==============================================================================
        :Unit: TNSComboBox
      :Author: Emil Lefkof, Consultant
                 Melman Consulting Group, Copyright 2000.
:Date Created: 01/26/2000
 :Description: This component is a combobox that will fill with available SID's
               from TNSNames.ora file if Oracle client software is installed.
               Useful for projects using ADO or BDE and want to let their users
               determine which Oracle instance to connect to.

               Homepage: http://www.melmanconsulting.com
               E-Mail:   emil.lefkof@melmanconsulting.com

               Notes: This component by default will look in the registry for the
                      location of the tnsnames.ora file.  If not found in the
                      registry it will look in C:\ORANT\NETWORK\ADMIN and
                      C:\ORAWAIN95\NETWORK\ADMIN respectively looking for a valid
                      tnsnames.ora file.  If none found then I am guessing ORacle
                      client software isn't installed.  However, if you store
                      tnsnames.ora file on a network drive etc. You can set the
                      Property TNSFilename and then call the method
                      LoadFromTNSNames at runtime to refill the combobox based on
                      that filename given.
============================================================================== }
unit TNSComboBox;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Registry;

type
  TTNSComboBox = class(TComboBox)
  private
    { Private declarations }
    FTNSNames : TStringList;
    FTNSFilename : string;
    procedure GetTNSNamesFileName();
    function GetTNSNamesFromFile() : TStringList;
  protected
    { Protected declarations }
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure LoadFromTNSNames;
    
    property TNSNames : TStringList read GetTNSNamesFromFile;
    property TNSFilename : string read FTNSFilename write FTNSFilename;
  published
    { Published declarations }
  end;

procedure Register;

implementation

{===============================================================================
  :Constructor: Create
       :Author: Emil Lefkof
 :Date Created: 01/26/2000
  :Description: Overridden constructor used to fill the combobox from tnsnames.ora
                only if not in Design Mode.
       :Inputs: Aowner(TComponent) The owner of this control

Modification History -----------------------------------------------------------

Date       Init  Description
----       ----  ---------------------------------------------------------------
01/26/2000 EAL   Intial Version
===============================================================================}
constructor TTNSComboBox.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  if not (csDesigning in componentstate) then begin
     GetTNSNamesFileName;
     LoadFromTNSNames();
  end;
end;

{===============================================================================
   :Destructor: Destroy
       :Author: Emil Lefkof
 :Date Created: 01/26/2000
  :Description: Overridden destructor used to free the stringlist holding the
                Oracle SID from the TNSNames.ora file.
       :Inputs: none

Modification History -----------------------------------------------------------

Date       Init  Description
----       ----  ---------------------------------------------------------------
01/26/2000 EAL   Intial Version
===============================================================================}
destructor TTNSComboBox.Destroy;
begin
  if FTNSNames <> nil then
     FTNSNames.free;
  inherited;
end;

{===============================================================================
    :Procedure: GetTNSNamesFileName
       :Author: Emil Lefkof
 :Date Created: 01/26/2000
  :Description: Tries to find the tnsnames.ora file to use.  First it checks
                for the location of the file in the registry. If not found then
                it will check both C:\ORANT\NETWORK\ADMIN and
                C:\ORAWIN95\NETWORK\ADMIN directories.  If not found in any of
                those places then just return nothing.
       :Inputs: none

Modification History -----------------------------------------------------------

Date       Init  Description
----       ----  ---------------------------------------------------------------
01/26/2000 EAL   Intial Version
24/08/2001 GSP   Modification for Oracle8i - Guilherme Siqueira Poli - guilherme@aganet.com
===============================================================================}
procedure TTNSComboBox.GetTNSNamesFileName();
   function TestOracleHome(Reg : TRegistry) : string;
   var
      strFilename : string;
   begin
      try
         strFilename := '';
         if Reg.OpenKeyReadOnly('\Software\Oracle') then
            strFilename := Trim(Reg.ReadString('ORACLE_HOME'));
         //if the str is not null then append the rest of the filename on
         if strFilename <> '' then begin
            strFilename := strFilename + '\network\admin\tnsnames.ora';
            //test if this file exists, if it doesn't then set the filename = ''
            if FileExists(strFilename) = False then
               strFilename := '';
            end;
         result := strFilename;
      except
         result := '';
      end;
   end;

var
  Reg: TRegistry;
  strFilename : string;
begin
  //first look for TNSNames.ora in the registry location
  Reg := TRegistry.Create;
  strFilename := '';
  try
    Reg.RootKey := HKEY_LOCAL_MACHINE;
    //try and locate TNS_ADMIN key
    if Reg.OpenKeyReadOnly('\Software\Oracle') then
       strFilename := Trim(Reg.ReadString('TNS_ADMIN'));

    //if not then use ORACLE_HOME to get the home directory and
    //look in /Network/Admin for tnsnames.ora
    if strFilename = '' then begin
       strFilename := TestOracleHome(Reg);
       end
    else begin
       //if the filename listed in TNS_ADMIN doesn't exist
       if FileExists(strFilename) = False then begin
          strFilename := TestOracleHome(Reg);
          end;
    end;
  finally
    Reg.CloseKey;
    Reg.Free;
  end;

  //if it wasn't found in the registry then try a couple of common places
  if strFilename = '' then begin
     //if it wasn't found in the registry then try C:\orant\network\admin
     if FileExists('C:\orant\network\admin\tnsnames.ora') then begin
        strFilename := 'C:\orant\network\admin\tnsnames.ora';
        end
     //if it wasn't found in the registry then try C:\orawin95\network\admin
     else if FileExists('C:\orawin95\network\admin\tnsnames.ora') then begin
        strFilename := 'C:\orawin95\network\admin\tnsnames.ora';
        end
     //Modification for Oracle8i
     else if FileExists('C:\oracle\ora81\network\ADMIN\tnsnames.ora') then begin
        strFilename := 'C:\oracle\ora81\network\ADMIN\tnsnames.ora';
        end
     else
        strFilename := '';
  end;

  //return the filename
  FTNSFilename := strFilename;
end;

{===============================================================================
    :Procedure: GetTNSNamesFromFile
       :Author: Emil Lefkof
 :Date Created: 01/26/2000
  :Description: If there is a valid tnsnames.ora the file exisits then parse the
                file and get the valid db instances from it and fill the stringlist.
       :Inputs: none

Modification History -----------------------------------------------------------

Date       Init  Description
----       ----  ---------------------------------------------------------------
01/26/2000 EAL   Intial Version
24/08/2001 GUI   Modification for Oracle8i - Guilherme Siqueira Poli - guilherme@aganet.com
===============================================================================}
function TTNSComboBox.GetTNSNamesFromFile() : TStringList;
var
   strLine : string;
   i       : Integer;
   noAlias : Boolean;
begin
   //if the TNSNames.ora file is empty then try and get it
   if FTNSFilename = '' then begin
      GetTNSNamesFileName();
      //if the name is still empty after trying to get it then it isn't
      //there so abort
      if FTNSFilename = '' then
         Abort;
      end;

   //check if the file actually exists
   if FileExists(FTNSFilename) = False then
      Abort;

   //if the stringlist is nil then create a new one else clear the list
   if FTNSNames = nil then
      FTNSNames := TStringList.Create
   else
      FTNSNames.Clear;

   try
      AssignFile(Input, FTNSFilename);
      Reset(Input);
      //loop through tnsnames.ora file parsing out valid SIDs
      while not SeekEof(Input) do begin
        Readln(Input, strLine);
        strLine := Trim(strLine);
        //if the line is one we should store then store it
        if Pos(' =',strLine) > 0 then begin
           strLine := Copy(strLine,0,Length(strLine)-2);
           //Modification for Oracle8i
           for i := 0 to Length(strLine) do
           begin
               if (strLine[i] = '(') then
                    noAlias := True;
           end;

           if not(noAlias) then
               FTNSNames.Add(strLine);
           end;

           noAlias := False;
      end;
      CloseFile(Input);
      if FTNSNames.Count <= 0 then
         Abort;
      FTNSNames.Sort;
      result := FTNSNames;
   except
      //MessageDlg('There was a problem reading the TNSnames.ora file.', mtError,[mbOK],0);
      FTNSNames.Free;
      result := nil;
   end;
end;

{===============================================================================
    :Procedure: LoadFromTNSNames
       :Author: Emil Lefkof
 :Date Created: 01/26/2000
  :Description: This method attempts to get the filename and fill the stringlist
                then fill the combo boxes items from that stringlist.  Should be
                called only externally if the user changed the value of the
                TNSFilename property.
       :Inputs: none

Modification History -----------------------------------------------------------

Date       Init  Description
----       ----  ---------------------------------------------------------------
01/26/2000 EAL   Intial Version
===============================================================================}
procedure TTNSComboBox.LoadFromTNSNames;
var
   i : integer;
begin
   if FTNSFilename <> '' then begin
      GetTNSNamesFromFile;
      if FTNSNames <> nil then begin
         Self.Items.Clear;
         for i := 0 to FTNSNames.Count - 1 do
            Self.Items.Add(FTNSNames[i]);
      end;
   end;
end;

{===============================================================================
    :Procedure: Register
       :Author: Emil Lefkof
 :Date Created: 01/26/2000
  :Description: Method used to register the control to the component palette.
                Will default to the Melman page unless this code here is changed.
       :Inputs: none

Modification History -----------------------------------------------------------

Date       Init  Description
----       ----  ---------------------------------------------------------------
01/26/2000 EAL   Intial Version
===============================================================================}
procedure Register;
begin
  RegisterComponents('Melman', [TTNSComboBox]);
end;

end.
